
# q-expansion of newform 49.5.d.b, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 49
weight = 5
poly_data = [484, 0, 22, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 0, 1, 0], 22], [[0, 0, 0, 1], 22]]

hecke_ring_character_values = [[3, [1, 0, 1, 0]]]
aps_data = [[-2, 1, -2, 0], [-2, 1, -1, -1], [5, -2, -5, -4], [0, 0, 0, 0], [0, 17, 29, 17], [70, 28, 140, 14], [82, -34, 41, 34], [-107, -37, 107, -74], [145, -41, 145, 0], [-544, 0, 0, 70], [-1206, 29, -603, -29], [-135, -104, -135, 0], [798, 84, 1596, 42], [618, 0, 0, -350], [257, 187, -257, 374], [0, -340, 2255, -340], [-842, 449, -421, -449], [47, -240, -47, -480], [0, 45, 659, 45], [-2602, 0, 0, 238], [-1738, -272, -869, 272], [-4055, 351, -4055, 0], [-1932, 168, -3864, 84], [-5665, 376, 5665, 752], [686, -2548, 1372, -1274], [19402, 50, 9701, -50], [4611, -1325, -4611, -2650], [4891, -461, 4891, 0], [0, -2398, -279, -2398], [-1888, 0, 0, -2366], [10418, 0, 0, 966], [3407, -317, -3407, -634], [0, 3230, -19879, 3230], [-6160, -1288, -12320, -644], [10603, 7078, 10603, 0], [0, -6451, 1485, -6451], [1118, 2752, 559, -2752], [-29591, 2871, -29591, 0], [-13566, 336, -27132, 168], [-21037, -128, 21037, -256], [0, -1663, 15737, -1663], [5712, 168, 11424, 84], [-31565, -4157, -31565, 0], [0, 4994, 31977, 4994], [3068, 0, 0, -4046], [-17054, 1569, -8527, -1569], [9690, 0, 0, -12866], [38556, -3864, 77112, -1932], [-48638, -1147, -24319, 1147], [-30403, -6246, 30403, -12492], [-401, -15266, -401, 0], [-11464, 0, 0, 4186], [49446, 4796, 24723, -4796], [-61362, -3024, -122724, -1512], [5045, 5542, -5045, 11084], [0, 19421, 40223, 19421], [13942, -13012, 6971, 13012], [5031, -1157, -5031, -2314], [0, -10224, -11101, -10224], [-72364, 0, 0, 4102], [-56534, 4593, -28267, -4593], [-9408, 9408, -18816, 4704], [12712, -9968, 25424, -4984], [39478, 16577, 19739, -16577], [43111, 10988, -43111, 21976], [-63695, 12412, -63695, 0], [46653, 379, 46653, 0], [42912, 0, 0, 18214], [0, -10399, -36469, -10399], [-15302, -8708, -30604, -4354], [-106094, 2066, -53047, -2066], [73435, -32969, 73435, 0], [173486, -11003, 86743, 11003], [8209, -2820, 8209, 0], [-154992, 0, 0, 24010], [-82399, 14047, 82399, 28094], [0, -46, -43819, -46], [63705, 7054, -63705, 14108], [39079, 35050, 39079, 0], [-63422, -15714, -31711, 15714], [-11424, -16800, -22848, -8400], [-28936, 0, 0, -8190], [0, 12449, -63139, 12449], [52346, 27524, 104692, 13762], [48865, -12553, -48865, -25106], [255463, 14239, 255463, 0], [112688, 0, 0, -10346], [-319167, -13628, -319167, 0], [-22974, 64428, -45948, 32214], [279302, 0, 0, 3906], [-160393, 2959, 160393, 5918], [208654, -895, 104327, 895], [0, 6653, 214887, 6653], [-199960, 0, 0, 13762], [189873, 2143, 189873, 0], [110124, -44856, 220248, -22428], [-21667, 42838, 21667, 85676], [-287954, 22520, -143977, -22520], [-120367, 8307, 120367, 16614], [-36689, -23358, -36689, 0], [-203992, 0, 0, -46662], [0, -6514, 249341, -6514], [150358, 2717, 75179, -2717], [240469, 47944, 240469, 0], [0, 112017, 32201, 112017], [-55946, -45828, -27973, 45828], [-156408, 11760, -312816, 5880], [15881, -74090, -15881, -148180], [0, -80959, -62425, -80959], [-85778, 96460, -171556, 48230], [222157, -5329, -222157, -10658], [0, 8298, -341767, 8298], [179804, 0, 0, 101962], [358062, 30605, 179031, -30605], [157586, 0, 0, -87738], [0, -18820, 359507, -18820], [-113596, -104944, -227192, -52472], [159094, -21895, 79547, 21895], [20557, -125096, 20557, 0], [-367582, 0, 0, -88130], [161726, -54368, 80863, 54368], [-584568, 0, 0, 3766], [138101, 11842, -138101, 23684], [0, 18245, -556051, 18245], [-228237, 9259, 228237, 18518], [-84502, 0, 0, -67928], [364005, 102796, 364005, 0], [250367, -14429, -250367, -28858], [24416, -180040, 48832, -90020], [-185061, 18856, 185061, 37712], [0, -24651, 539729, -24651], [554906, 0, 0, -49490], [-304509, -63797, -304509, 0], [163284, 0, 0, 65338], [1013, -15122, -1013, -30244], [211806, 24276, 423612, 12138], [269218, -1042, 134609, 1042], [-1097210, 21561, -548605, -21561], [401478, -44268, 802956, -22134], [0, 132464, 76595, 132464], [118524, 244272, 237048, 122136], [-255593, 68230, -255593, 0], [0, -42039, 238799, -42039], [333566, 0, 0, 49882], [-335302, 28414, -167651, -28414], [349356, 44016, 698712, 22008], [-251706, -57876, -503412, -28938], [-775154, -76, -387577, 76], [-267507, -38201, 267507, -76402], [-5819, 2311, -5819, 0], [343915, -30330, 343915, 0], [261072, -102312, 522144, -51156], [-489690, 0, 0, 126406], [508037, -1325, -508037, -2650], [0, -188143, -341865, -188143], [483170, 0, 0, 132874], [-339453, -54977, -339453, 0], [-644485, 2392, 644485, 4784], [-78904, -102592, -157808, -51296], [383710, -107596, 191855, 107596], [-14303, -209789, -14303, 0], [120248, 0, 0, 95242], [220670, 0, 0, -299418], [142511, 111067, -142511, 222134], [0, 170978, -209719, 170978], [1173058, 43709, 586529, -43709], [0, 157433, 780039, 157433], [563974, 109236, 281987, -109236]]
